const program = require('commander');
const prompt = require('prompt');
const Progress = require('progress');
const Builder = require('./utils/builder');
const ResManager = require('./utils/resManager');
const Fix = require('./utils/fix');

const pjson = require('./package');
program.version(pjson.version);

function getManyResources(group, resource) {
    if (group) {
        if (resource) {
            return [{ group, resource }];
        }
        return ResManager.getGroupResources(group);
    }
    return ResManager.getAllResources();
}

program
    .command('build [group] [resource]')
    .alias('b')
    .description('builds resources')
    .action((group, resource) => {
        const manyResources = getManyResources(group, resource);
        const bar = new Progress('[:bar] :percent completed ETA: :etas', { total: manyResources.length });
        Builder.buildMany(manyResources, () => {
            bar.tick();
        }, () => {
            console.log('complete');
        });
    });

program
    .command('watch')
    .alias('w')
    .description('watches for changes and rebuilds resources')
    .action(() => {
        var builder = new Builder();
        builder.watch();
    });

program
    .command('generate [group] [name]')
    .alias('g')
    .description('generates typescript structure for resources')
    .action((group, resource) => {
        const manyResources = getManyResources(group, resource);
        manyResources.forEach((resourceData) => {
            if (Builder.isGenerated(resourceData.group, resourceData.resource)) {
                if (manyResources.length === 1) {
                    console.log(`Typescript structure for ${resourceData.group}/${resourceData.resource} is already generated`);
                    console.log('Do you want to override it? (y/n)');
                    prompt.get(['override'], (err, result) => {
                        if (result.override === 'y') {
                            Builder.generate(resourceData.group, resourceData.resource);
                        }
                    });
                }
            }
            else {
                Builder.generate(resourceData.group, resourceData.resource);
            }
        });
    });

program
    .command('fix')
    .description('Fixes some packages after npm install')
    .action(() => {
        const fix = new Fix();
        fix.run();
    });

program.parse(process.argv);
