const fs = require('fs');
const parseRegex = new RegExp(/lib\.(\w+) = function\((?:mode|options|\))((?:(?!\.prototype = )[.\S\s])+)\.prototype = ([^\n]+)/, 'g');

module.exports = class {
  constructor(sourceDir, name) {
    this.sourceDir = sourceDir;
    this.name = name;
  }

  parse() {
    const types = {};
    if (!fs.existsSync(`${this.sourceDir}${this.name}.js`)) {
      return 0;
    }
    const content = fs.readFileSync(`${this.sourceDir}${this.name}.js`, 'UTF-8');
    console.log('content', content);
    
    let match = parseRegex.exec(content);
    while (match !== null) {
      const properties = {};
      const propRegex = new RegExp(/this\.(\w+) = new (cjs|lib|\$)\.(\w+)/, 'g');
      let propMatch = propRegex.exec(match[2]);
      while (propMatch !== null) {
        properties[propMatch[1]] = this.getType(propMatch[2], propMatch[3]);
        propMatch = propRegex.exec(match[2]);
      }
      const extend = this.getExtend(match[1], match[3]);

      types[match[1]] = {
        properties,
        extend
      };

      match = parseRegex.exec(content);
    }

    this.write(types);
    return Object.keys(types).length;
  }

  getType(namespace, type) {
    if (namespace === 'cjs') {
      return `createjs.${type}`;
    }
    if (namespace === 'lib') {
      return type;
    }
    return 'any';
  }

  write(types) {
    const lines = [];
    lines.push('/// <reference types="createjs" />');
    lines.push(`declare namespace ${this.name} {`);
    for (let typeName in types) {
      if (!types.hasOwnProperty(typeName)) {
        continue;
      }
      const type = types[typeName];
      let typeKey = typeName;
      if (typeKey === this.name) {
        typeKey = 'root';
      }
      lines.push(`  export interface ${typeKey} extends ${type.extend.join(', ')} {`);
      lines.push(`    new(): ${typeKey};`);
      for (let propName in type.properties) {
        if (!type.properties.hasOwnProperty(propName)) {
          continue;
        }
        const propType = type.properties[propName];
        lines.push(`    ${propName}: ${propType};`);
      }
      lines.push('  }');
    }
    lines.push('  export interface lib {');
    for (let typeName in types) {
      if (!types.hasOwnProperty(typeName)) {
        continue;
      }
      let typeKey = typeName;
      if (typeKey === this.name) {
        typeKey = 'root';
      }
      lines.push(`    ${typeName}: ${typeKey};`);
    }
    lines.push('  }');
    lines.push('}');
    lines.push(`declare interface ${this.name} {`);
    lines.push(`  root: ${this.name}.root;`);
    lines.push('}');
    fs.writeFileSync(`${this.sourceDir}index.d.ts`, lines.join('\n'));
  }

  appendType(types, type, extendType) {
    if (type !== extendType) {
      types.push(extendType);
    }
  }

  getExtend(type, lastLine) {
    const mixedRegex = new RegExp(/getMCSymbolPrototype\([new\ ]*(lib|cjs)\.(\w+), [new\ ]*(lib|cjs)\.(\w+)[^\n]*/, 'g');
    const singleRegex = new RegExp(/p = new (lib|cjs)\.(\w+)/, 'g');

    let extend = ['createjs.MovieClip'];
    const matchType = mixedRegex.exec(lastLine);
    const singleMatch = singleRegex.exec(lastLine);
    if (matchType !== null) {
      this.appendType(extend, type, this.getType(matchType[1], matchType[2]));
      this.appendType(extend, type, this.getType(matchType[3], matchType[4]));
    }
    else if (singleMatch !== null) {
      extend = [this.getType(singleMatch[1], singleMatch[2])];
    }
    return extend;
  }
};
